"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var utils_1 = require("./utils");
var typescript_1 = require("typescript");
function parseConfigFile(filePath) {
    var _a = typescript_1.readConfigFile(filePath, utils_1.readFile), config = _a.config, error = _a.error;
    if (utils_1.isNil(error)) {
        if (!utils_1.isNil(config.extends)) {
            var extendFrom = config.extends;
            var baseConfigPath = path.resolve(path.dirname(filePath), extendFrom);
            if (utils_1.isFile(baseConfigPath) && baseConfigPath !== filePath) {
                var baseConfigs = parseConfigFile(baseConfigPath);
                if (utils_1.isNil(baseConfigs.error)) {
                    delete config.extends;
                    var extendedConfigs = utils_1.mergeInto(config, baseConfigs.configs);
                    return {
                        configs: extendedConfigs,
                        error: undefined,
                    };
                }
            }
        }
    }
    return {
        configs: config,
        error: error,
    };
}
exports.parseConfigFile = parseConfigFile;
function getSafePackageName(name) {
    if (/^(?:[a-z][a-z0-9._-]+?|@[^\/]*\/\b)+$/.test(name) === false) {
        return undefined;
    }
    return name.replace(/[^\w-]/g, '-').replace(/^-+/, '');
}
exports.getSafePackageName = getSafePackageName;
function createNGCConfig(filePath, moduleId, configs) {
    var result = utils_1.mergeInto({
        compilerOptions: {
            outDir: './ngc-compiled',
            baseUrl: '.',
            target: 'es2015',
            module: 'es2015',
            sourceMap: false,
            declaration: true,
            lib: [
                'es2017',
                'dom',
            ],
        },
        angularCompilerOptions: {
            strictMetadataEmit: true,
            skipTemplateCodegen: true,
            annotateForClosureCompiler: true,
            flatModuleOutFile: moduleId + '.js',
        },
    }, configs);
    if (!utils_1.isNil(result.angularCompilerOptions) && utils_1.isNil(result.angularCompilerOptions.flatModuleId)) {
        result.angularCompilerOptions.flatModuleId = moduleId;
    }
    if (!utils_1.isNil(result.include)) {
        delete result.include;
    }
    if (!utils_1.isNil(result.exclude)) {
        delete result.exclude;
    }
    var resultString = JSON.stringify(result);
    utils_1.writeFile(filePath, resultString);
}
exports.createNGCConfig = createNGCConfig;
function readPackage(packageFilePath) {
    var content = utils_1.readFile(packageFilePath);
    try {
        return JSON.parse(content);
    }
    catch (_err) {
        return {};
    }
}
exports.readPackage = readPackage;
function isPkgModuleEntryValid(value, entry, projectPath) {
    value = path.resolve(projectPath, value);
    entry = path.resolve(projectPath, entry);
    return value.toLowerCase() === entry.toLowerCase();
}
function validatePkgModuleEntries(_a, pkg, projectPath) {
    var pkgMain = _a.pkgMain, pkgModule = _a.pkgModule, pkgES2015 = _a.pkgES2015, pkgTypings = _a.pkgTypings;
    if (!utils_1.isNil(pkgMain)) {
        if (utils_1.isNil(pkg.main) || !isPkgModuleEntryValid(pkgMain, pkg.main, projectPath)) {
            console.warn("Warning: You must set \"main\" property of your package.json to \"" + pkgMain.replace(projectPath + path.sep, '') + "\"");
        }
    }
    if (!utils_1.isNil(pkgModule)) {
        if (utils_1.isNil(pkg.module) || !isPkgModuleEntryValid(pkgModule, pkg.module, projectPath)) {
            console.warn("Warning: You must set \"module\" property of your package.json to \"" + pkgModule.replace(projectPath + path.sep, '') + "\"");
        }
    }
    if (!utils_1.isNil(pkgES2015)) {
        if (utils_1.isNil(pkg.es2015) || !isPkgModuleEntryValid(pkgES2015, pkg.es2015, projectPath)) {
            console.warn("Warning: You must set \"es2015\" property of your package.json to \"" + pkgES2015.replace(projectPath + path.sep, '') + "\"");
        }
    }
    if (!utils_1.isNil(pkgTypings)) {
        if (utils_1.isNil(pkg.typings) || !isPkgModuleEntryValid(pkgTypings, pkg.typings, projectPath)) {
            console.warn("Warning: You must set \"typings\" property of your package.json to \"" + pkgTypings.replace(projectPath + path.sep, '') + "\"");
        }
    }
}
exports.validatePkgModuleEntries = validatePkgModuleEntries;
//# sourceMappingURL=config-helpers.js.map