"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var utils_1 = require("./utils");
var FileHandler = (function () {
    function FileHandler(filePath) {
        this.filePath = filePath;
        this.importModuleRE = /(?:import|export)\s*(?:(?:\{[^}]*\}|\*|\w+)(?:\s*as\s+\w+)?(?:\s+from)?\s*)?([`'"])((?:\\[\s\S]|(?!\1)[^\\])*?)\1/ig;
        this.dirPath = path.dirname(filePath);
    }
    Object.defineProperty(FileHandler.prototype, "content", {
        get: function () {
            if (utils_1.isNil(this._content)) {
                this._content = utils_1.readFile(this.filePath);
            }
            return this._content;
        },
        enumerable: true,
        configurable: true
    });
    FileHandler.prototype.resolveToFile = function (validPath) {
        if (utils_1.isDirectory(validPath)) {
            validPath = path.join(validPath, 'index.ts');
        }
        else if (path.extname(validPath) !== '.ts') {
            validPath += '.ts';
        }
        return validPath;
    };
    FileHandler.prototype.getImporteeNames = function () {
        var allModules = [];
        var importModuleMatchFileIndex = 2;
        var importModuleMatch;
        // Reset current index to 0
        // RegEx.test moves this into 1
        this.importModuleRE.lastIndex = 0;
        var content = this.content;
        importModuleMatch = this.importModuleRE.exec(content);
        while (!utils_1.isNil(importModuleMatch)) {
            /**
             * Adds the file from match to result
             * import * as x from './a-module'; => will get "./a-module"
             */
            allModules.push(importModuleMatch[importModuleMatchFileIndex]);
            importModuleMatch = this.importModuleRE.exec(content);
        }
        return allModules;
    };
    FileHandler.prototype.getImporteeFiles = function (excludeFrom) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getImporteeNames()
                            .map(function (moduleName) { return path.resolve(_this.dirPath, moduleName); })
                            .map(function (resolvedPath) { return _this.resolveToFile(resolvedPath); })
                            .filter(function (resolvedFilePath) { return utils_1.isFile(resolvedFilePath); })
                            .filter(function (resolvedFilePath) { return (utils_1.isEmpty(excludeFrom) || excludeFrom.indexOf(resolvedFilePath) < 0); })
                            .map(function (resolvedFilePath) { return new FileHandler(resolvedFilePath); })];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    FileHandler.prototype.setContent = function (content) {
        this._content = content;
    };
    FileHandler.prototype.copyTo = function (destFilePath) {
        var dirPath = path.dirname(destFilePath);
        utils_1.ensureMakeDir(dirPath);
        utils_1.writeFile(destFilePath, this.content);
    };
    FileHandler.prototype.hasImporters = function () {
        this.importModuleRE.lastIndex = 0;
        return this.importModuleRE.test(this.content);
    };
    FileHandler.prototype.getImportees = function (excludeFrom) {
        return __awaiter(this, void 0, void 0, function () {
            var importeeFiles, allImportees, _i, importeeFiles_1, file, currentFileImportees;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getImporteeFiles(excludeFrom)];
                    case 1:
                        importeeFiles = _a.sent();
                        if (utils_1.isEmpty(importeeFiles)) {
                            return [2 /*return*/, []];
                        }
                        excludeFrom.push.apply(excludeFrom, importeeFiles.map(function (file) { return file.filePath; }));
                        allImportees = importeeFiles.slice();
                        _i = 0, importeeFiles_1 = importeeFiles;
                        _a.label = 2;
                    case 2:
                        if (!(_i < importeeFiles_1.length)) return [3 /*break*/, 5];
                        file = importeeFiles_1[_i];
                        return [4 /*yield*/, file.getImportees(excludeFrom)];
                    case 3:
                        currentFileImportees = _a.sent();
                        if (!utils_1.isEmpty(currentFileImportees)) {
                            allImportees.push.apply(allImportees, currentFileImportees);
                            excludeFrom.push.apply(excludeFrom, currentFileImportees.map(function (moduleFile) { return moduleFile.filePath; }));
                        }
                        _a.label = 4;
                    case 4:
                        _i++;
                        return [3 /*break*/, 2];
                    case 5: return [2 /*return*/, allImportees];
                }
            });
        });
    };
    return FileHandler;
}());
exports.FileHandler = FileHandler;
//# sourceMappingURL=file.js.map