"use strict";
/**
 * @license Utils
 * (c) 2017 Amin Paks <amin.pakseresht@hotmail.com>
 * License: MIT
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var glob = require("glob");
var path = require("path");
function copyFromTo(_a) {
    var pattern = _a.pattern, rootDir = _a.rootDir, toDir = _a.toDir, _b = _a.excludes, excludes = _b === void 0 ? [] : _b;
    return new Promise(function (resolve, reject) {
        glob(pattern, {
            cwd: rootDir,
            nodir: true,
            ignore: ['node_modules/**/*'].concat(excludes),
        }, function (err, files) {
            // Rejects on error
            if (err) {
                reject(err);
            }
            var results = files.map(function (file) {
                return new Promise(function (fileResolve, fileReject) {
                    var origin = path.join(rootDir, file);
                    var dest = path.join(toDir, file);
                    var rd = fs.createReadStream(origin);
                    rd.on('error', function (err_1) { return fileReject("Cannot read " + origin + ": " + err_1); });
                    rd.on('open', function () {
                        ensureMakeDir(path.dirname(dest));
                        var wr = fs.createWriteStream(dest);
                        wr.on('error', function (err_1) { return fileReject("Cannot write to " + dest + ": " + err_1); });
                        wr.on('close', function () { return fileResolve(file); });
                        // Finish the copy
                        rd.pipe(wr);
                    });
                });
            });
            Promise.all(results)
                .then(function (values) {
                if (values.length === files.length) {
                    resolve(values);
                }
                else {
                    var notCopied = files.filter(function (file) { return !values.includes(file); });
                    reject('Not all files were copied.\n' + notCopied.join('\n'));
                }
            })
                .catch(function (err_1) {
                reject(err_1);
            });
        });
    });
}
exports.copyFromTo = copyFromTo;
function readFile(filePath, encoding) {
    return fs.readFileSync(filePath, { encoding: encoding || 'utf8' });
}
exports.readFile = readFile;
function writeFile(filePath, content, encoding) {
    return fs.writeFileSync(filePath, content, { encoding: encoding || 'utf8' });
}
exports.writeFile = writeFile;
function isDirectory(validPath) {
    try {
        return fs.lstatSync(validPath).isDirectory();
    }
    catch (err) {
        return false;
    }
}
exports.isDirectory = isDirectory;
function isFile(validPath) {
    try {
        return fs.lstatSync(validPath).isFile();
    }
    catch (err) {
        return false;
    }
}
exports.isFile = isFile;
function isString(value) {
    return (typeof value === 'string');
}
exports.isString = isString;
function isNil(value) {
    return (value == undefined);
}
exports.isNil = isNil;
function isEmpty(value) {
    if (typeof value === 'string') {
        return value.length === 0;
    }
    else if (value instanceof Array) {
        return value.length === 0;
    }
    else if (typeof value === 'object') {
        return Object.keys(value).length === 0;
    }
    else {
        return true;
    }
}
exports.isEmpty = isEmpty;
function isArray(variable) {
    return (variable instanceof Array);
}
exports.isArray = isArray;
function ensureMakeDir(validPath) {
    if (!isDirectory(validPath)) {
        var parentPath = path.dirname(validPath);
        ensureMakeDir(parentPath);
        fs.mkdirSync(validPath);
    }
}
exports.ensureMakeDir = ensureMakeDir;
function ensureRemoveDir(validPath) {
    if (isDirectory(validPath)) {
        var dirItems = fs.readdirSync(validPath, 'utf8');
        for (var _i = 0, dirItems_1 = dirItems; _i < dirItems_1.length; _i++) {
            var item = dirItems_1[_i];
            var currentItem = path.resolve(validPath, item);
            if (isDirectory(currentItem)) {
                ensureRemoveDir(currentItem);
            }
            else if (isFile(currentItem)) {
                fs.unlinkSync(currentItem);
            }
        }
        fs.rmdirSync(validPath);
    }
}
exports.ensureRemoveDir = ensureRemoveDir;
function mergeLevelFromTo(from, to) {
    var result = __assign({}, to);
    var _loop_1 = function (key) {
        if (isArray(from[key])) {
            var fromArray_1 = from[key];
            if (isArray(result[key])) {
                var toArray = result[key];
                result[key] = toArray
                    .filter(function (item) { return fromArray_1.indexOf(item) < 0; })
                    .concat(fromArray_1);
            }
            else {
                result[key] = fromArray_1.slice();
            }
        }
        else if (typeof from[key] === 'object') {
            result[key] = mergeLevelFromTo(from[key], result[key]);
        }
        else {
            result[key] = from[key];
        }
    };
    for (var _i = 0, _a = Object.keys(from); _i < _a.length; _i++) {
        var key = _a[_i];
        _loop_1(key);
    }
    return result;
}
exports.mergeLevelFromTo = mergeLevelFromTo;
function mergeInto(origin, dest) {
    return mergeLevelFromTo(origin, dest);
}
exports.mergeInto = mergeInto;
//# sourceMappingURL=utils.js.map